# kim/pipelines/string.py
# Copyright (C) 2014-2015 the Kim authors and contributors
# <see AUTHORS file>
#
# This module is part of Kim and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

import six

from .base import pipe, is_valid_choice
from .marshaling import MarshalPipeline
from .serialization import SerializePipeline


@pipe()
def bounds_check(session):
    """Pipe used to determine if a value is within the min and max bounds on
    the field

    :param session: Kim pipeline session instance

    """

    max_ = session.field.opts.max
    min_ = session.field.opts.min

    if max_ is not None and len(session.data) > max_:
        raise session.field.invalid(error_type='out_of_bounds')
    if min_ is not None and len(session.data) < min_:
        raise session.field.invalid(error_type='out_of_bounds')

    return session.data


@pipe()
def is_valid_string(session):
    """Pipe used to determine if a value can be coerced to a string

    :param session: Kim pipeline session instance
    """

    try:
        session.data = six.text_type(session.data)
        return session.data
    except ValueError:
        raise session.field.invalid(error_type='type_error')


@pipe()
def blank_check(session):
    """Pipe used to determine if a value is blank. If blank=False and value
    is the empty string, raise error

    :param session: Kim pipeline session instance
    """

    if session.data == '' and session.field.opts.blank is False:
        raise session.field.invalid(error_type='type_error')

    return session.data


@pipe(run_if_none=False)
def to_unicode(session):
    """Convert incoming value to unicode string

    :param session: Kim pipeline session instance
    """
    return six.text_type(session.data)


class StringMarshalPipeline(MarshalPipeline):
    """StringMarshalPipeline

    .. seealso::
        :func:`kim.pipelines.base.is_valid_choice`
        :func:`kim.pipelines.string.is_valid_string`
        :class:`kim.pipelines.marshaling.MarshalPipeline`
    """

    validation_pipes = \
        [is_valid_string, blank_check, is_valid_choice, bounds_check] \
        + MarshalPipeline.validation_pipes

    output_pipes = [to_unicode] + MarshalPipeline.output_pipes


class StringSerializePipeline(SerializePipeline):
    """StringSerializePipeline

    .. seealso::
        :class:`kim.pipelines.serialization.SerializePipeline`
    """
    pass



# kim/field.py
# Copyright (C) 2014-2015 the Kim authors and contributors
# <see AUTHORS file>
#
# This module is part of Kim and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from collections import defaultdict

from .exception import FieldError, FieldInvalid, FieldOptsError
from .utils import set_creation_order
from .pipelines import (
    StringMarshalPipeline, StringSerializePipeline,
    StaticSerializePipeline,
    IntegerMarshalPipeline, IntegerSerializePipeline,
    NestedMarshalPipeline, NestedSerializePipeline,
    CollectionMarshalPipeline, CollectionSerializePipeline,
    BooleanMarshalPipeline, BooleanSerializePipeline,
    DateTimeSerializePipeline, DateTimeMarshalPipeline,
    DateMarshalPipeline, DateSerializePipeline,
    DecimalSerializePipeline, DecimalMarshalPipeline,
    FloatSerializePipeline, FloatMarshalPipeline)
from .pipelines.base import run_pipeline, Session
from .pipelines.marshaling import MarshalPipeline
from .pipelines.serialization import SerializePipeline

DEFAULT_ERROR_MSGS = {
    'required': 'This is a required field',
    'type_error': 'Invalid type',
    'not_found': '{name} not found',
    'none_not_allowed': 'This field cannot be null',
    'invalid_choice': 'invalid choice',
    'duplicates': 'duplicates found',
    'out_of_bounds': 'value out of allowed range',
}


class FieldOpts(object):
    """FieldOpts are used to provide configuration options to :class:`.Field`.
    They are designed to allow users to easily provide custom configuration
    options to :class:`.Field` classes.

    Custom :class:`.FieldOpts` classes are set on :class:`.Field` using the
    ``opts_class`` property.

    .. code-block:: python

        class MyFieldOpts(FieldOpts):

            def __init__(self, **opts):

                self.some_property = opts.get('some_property', None)
                super(MyFieldOpts, self).__init__(**opts)

    .. seealso::
        :class:`.Field`
    """

    extra_error_msgs = {}

    def __init__(self, **opts):
        """ Construct a new instance of :class:`FieldOpts`
        and set config options

        :param name: Specify the name of the field for data output
        :param required: This field must be present when marshaling
        :param attribute_name: Specify internal name for this field, set on
            mapper.fields dict
        :param source: Specify the name of the attribute on the object to use
            when getting/setting data. May be ``__self__`` to use entire mapper
            object as data
        :param default: Specify a default value for this field to apply when
            serializing or marshaling
        :param allow_none: This option only takes affect if required=False. If
            allow_none=False and required=False, then Kim will accept either
            the field being missing completely from the data, or the field
            being passed with a non-None value. That is, either ``{}`` or
            ``{'field': 'value'}`` but never ``{'field': None}``. Default True.
        :param read_only: Specify if this field should be ignored when marshaling
        :param error_msgs: A dict of error_type: error messages.
        :param null_default: Specify the default type to return when a field is
            null IE None or {} or ''
        :param choices: Specify a list of valid values
        :param extra_serialize_pipes: dict of lists containing extra Pipe functions
            to be run at the end of each stage when serializing.
            eg ``{'output': [my_pipe, my_other_pipe]}```
        :param extra_marshal_pipes: dict of lists containing extra Pipe functions
            to be run at the end of each stage when marshaling.
            eg ``{'validate': [my_pipe, my_other_pipe]}```

        :raises: :class:`.FieldOptsError`
        :returns: None
        """

        self._opts = opts.copy()

        # internal attrs
        self._is_wrapped = opts.pop('_is_wrapped', False)

        # set attribute_name, name and source options.
        name = opts.pop('name', None)
        attribute_name = opts.pop('attribute_name', None)
        source = opts.pop('source', None)

        self.name, self.attribute_name, self.source = None, None, None

        self.set_name(name=name, attribute_name=attribute_name, source=source)

        self.error_msgs = DEFAULT_ERROR_MSGS.copy()
        self.error_msgs.update(opts.pop('error_msgs', self.extra_error_msgs))

        self.required = opts.pop('required', True)
        self.default = opts.pop('default', None)
        self.null_default = opts.pop('null_default', None)

        self.allow_none = opts.pop('allow_none', True)
        self.read_only = opts.pop('read_only', False)
        self.choices = opts.pop('choices', None)

        self.extra_marshal_pipes = \
            opts.pop('extra_marshal_pipes', defaultdict(list))
        self.extra_serialize_pipes = \
            opts.pop('extra_serialize_pipes', defaultdict(list))

        self.validate()

    def validate(self):
        """Allow users to perform checks for required config options.  Concrete
        classes should raise :class:`.FieldError` when invalid configuration
        is encountered.

        A slightly contrived example is requiring all fields to be
        ``read_only=True``

        Usage::

            from kim.field import FieldOpts

            class MyOpts(FieldOpts):

                def validate(self):

                    if self.read_only is True:
                        raise FieldOptsError('Field cannot be read only')


        :raises: `.FieldOptsError`
        :returns: None
        """
        pass

    def set_name(self, name=None, attribute_name=None, source=None):
        """Programmatically set the name properties for a field.

        Names cascade from each other unless they are explicitly overridden.

        Example 1:
        class MyMapper(Mapper):
            foo = field.String()

        attribute_name = foo
        name = foo
        source = foo

        Example 2:
        class MyMapper(Mapper):
            foo = field.String(name='bar', source='baz')

        attribute_name = foo
        name = bar
        source = baz

        :param name: value of name property
        :param attribute_name: value of attribute_name property
        :param source: value of source property

        :returns: None
        """
        self.attribute_name = self.attribute_name or attribute_name
        self.name = self.name or name or self.attribute_name
        self.source = self.source or source or self.name

    def get_name(self):
        """Return the name property set by :meth:`set_name`

        :rtype: str
        :returns: the name of the field to be used in input/output
        """

        return self.name


class Field(object):
    """Field, as it's name suggests, represents a single key or 'field'
    inside of your mappings.  Much like columns in a database or a csv,
    they provide a way to represent different data types when pushing data
    into and out of your Mappers.

    A core concept of Kims architecture is that of Pipelines.
    Every Field makes use of both an Input and Output pipeline which affords users
    a great level of flexibility when it comes to handling data.

    Kim provides a collection of default Field implementations,
    for more complex cases extending Field to create new field types
    couldn't be easier.

    Usage::

        from kim import Mapper
        from kim import field

        class UserMapper(Mapper):

            id = field.Integer(required=True, read_only=True)
            name = field.String(required=True)
    """

    #: The :class:`FieldOpts` field config class to use for the Field.
    opts_class = FieldOpts

    #: The Fields marshaling pipeline
    marshal_pipeline = MarshalPipeline

    #: The Fields serialization pipeline
    serialize_pipeline = SerializePipeline

    def __init__(self, *args, **field_opts):
        """Constructs a new instance of Field.  Each Field accepts a set of
        kwargs that will be passed directly to the fields
        defined :class:`FieldOpts`.

        :param args: list of arguments passed to the field
        :param kwargs: keyword arguments typically passed to the FieldOpts class attached
            to this Field.
        :raises: :class:`FieldOptsError`
        :returns: None

        .. seealso::
            :class:`FieldOpts`
        """

        try:
            self.opts = self.opts_class(*args, **field_opts)
        except FieldOptsError as e:
            msg = '{0} field has invalid options: {1}' \
                .format(self.__class__.__name__, e.message)
            raise FieldError(msg)

        set_creation_order(self)

        self.marshal_pipes = self.marshal_pipeline.get_pipeline(
            **self.opts.extra_marshal_pipes
        )
        self.serialize_pipes = self.serialize_pipeline.get_pipeline(
            **self.opts.extra_serialize_pipes
        )

    def get_error(self, error_type):
        """Return the error message for ``error_type`` from the error messages defined on
        the fields opts class.

        :param error_type: the key of the error found in self.error_msgs
        :returns: Error message
        :rtype: string
        """

        parse_opts = {
            'name': self.name
        }
        return self.opts.error_msgs[error_type].format(**parse_opts)

    def invalid(self, error_type):
        """Raise an Exception using the provided error_type for the error message.
        This method is typically used by pipes to allow :class:`Field` to control
        how its errors are handled.

        Usage::

            @pipe()
            def validate_name(session):
                if session.data and session.data != 'Mike Waites':
                    raise session.fied.invalid('not_mike')

        :param error_type: The key of the error being raised.
        :raises: :class:`FieldInvalid`

        .. seealso::
            :class:`FieldOpts` for an explanation on defining error messags
        """

        raise FieldInvalid(self.get_error(error_type), field=self)

    @property
    def name(self):
        """Proxy access to the :class:`FieldOpts` defined for this field.

        :rtype: str
        :returns: The value of get_name from FieldOpts
        :raises: :class:`FieldError`

        .. seealso::
            :meth:`kim.field.FieldOpts.get_name`
        """

        field_name = self.opts.get_name()
        if not field_name:
            cn = self.__class__.__name__
            raise FieldError('{0} requires {0}.name or '
                             '{0}.attribute_name.  Please provide a `name` '
                             'or `attribute_name` param to {0}'.format(cn))

        return field_name

    @name.setter
    def name(self, name):
        """Proxy setting the name property via :py:meth:`kim.field.FieldOpts.set_name`

        :param name: the value to set against FieldOpts name property
        :returns: None

        .. seealso::
            :meth:`kim.field.FieldOpts.set_name`
        """
        self.opts.set_name(name)

    def marshal(self, mapper_session, **opts):
        """Run the marshal :class:`Pipeline` for this field for the given ``data`` and
        update the output for this field inside of the mapper_session.

        :param mapper_session: The Mappers marshaling session this field is being
            run inside of.
        :opts: kwargs passed to the marshal pipelines run method.
        :returns: None

        .. seealso::
            :meth:`kim.mapper.Mapper.marshal`
        """

        parent = opts.get('parent_session', None)
        session = Session(
            self, mapper_session.data, mapper_session.output,
            mapper_session=mapper_session,
            parent=parent)
        run_pipeline(self.marshal_pipes, session, self, **opts)

    def serialize(self, mapper_session, **opts):
        """Run the serialize :class:`Pipeline` for this field for the given `data` and
        update `output` in for this field inside of the mapper_session.

        :param mapper_session: The Mappers marshaling session this field is being
            run inside of.
        :opts: kwargs passed to the marshal pipelines run method.
        :returns: None

        .. seealso::
            :meth:`kim.mapper.Mapper.serialize`
        """
        parent = opts.get('parent_session', None)
        session = Session(
            self, mapper_session.data, mapper_session.output,
            mapper_session=mapper_session,
            parent=parent)

        run_pipeline(self.serialize_pipes, session, self, **opts)


class StringFieldOpts(FieldOpts):
    """Custom FieldOpts class that provides additional config options for
    :class:`String`.

    """

    def __init__(self, **kwargs):
        """ Construct a new instance of :class:`StringFieldOpts`
        and set config options

        :param max: Specify the maximum permitted length
        :param min: Specify the minimum permitted length
        :param blank: If False, raise error if empty string passed. Default True

        :raises: :class:`FieldOptsError`
        :returns: None
        """
        self.max = kwargs.pop('max', None)
        self.min = kwargs.pop('min', None)
        self.blank = kwargs.pop('blank', True)
        super(StringFieldOpts, self).__init__(**kwargs)


class String(Field):
    """:class:`String` represents a value that must be valid
    when passed to str()

    Usage::

        from kim import Mapper
        from kim import field

        class UserMapper(Mapper):
            __type__ = User

            name = field.String(required=True)

    """
    opts_class = StringFieldOpts
    marshal_pipeline = StringMarshalPipeline
    serialize_pipeline = StringSerializePipeline


class IntegerFieldOpts(FieldOpts):
    """Custom FieldOpts class that provides additional config options for
    :class:`Integer`.

    """

    def __init__(self, **kwargs):
        """ Construct a new instance of :class:`IntegerFieldOpts`
        and set config options

        :param max: Specify the maximum permitted value
        :param min: Specify the minimum permitted value

        :raises: :class:`FieldOptsError`
        :returns: None
        """
        self.max = kwargs.pop('max', None)
        self.min = kwargs.pop('min', None)
        super(IntegerFieldOpts, self).__init__(**kwargs)


class Integer(Field):
    """:class:`Integer` represents a value that must be valid
    when passed to int()

    Usage::

        from kim import Mapper
        from kim import field

        class UserMapper(Mapper):
            __type__ = User

            id = field.Integer(required=True, min=1, max=10)

    """

    opts_class = IntegerFieldOpts
    marshal_pipeline = IntegerMarshalPipeline
    serialize_pipeline = IntegerSerializePipeline


class FloatFieldOpts(FieldOpts):
    """Custom FieldOpts class that provides additional config options for
    :class:`Float`.

    """

    def __init__(self, **kwargs):
        """ Construct a new instance of :class:`FloatFieldOpts`
        and set config options


        :param precision: Specify the precision of the float
        :param max: Specify the maximum permitted value
        :param min: Specify the minimum permitted value

        :raises: :class:`FieldOptsError`
        :returns: None
        """
        self.precision = kwargs.pop('precision', 5)
        self.max = kwargs.pop('max', None)
        self.min = kwargs.pop('min', None)
        super(FloatFieldOpts, self).__init__(**kwargs)


class Float(Field):
    """:class:`Float` represents a value that must be valid
    Float type.

    Usage::

        from kim import Mapper
        from kim import field

        class UserMapper(Mapper):
            __type__ = User

            score = field.Float(precision=4)

    """

    opts_class = FloatFieldOpts
    marshal_pipeline = FloatMarshalPipeline
    serialize_pipeline = FloatSerializePipeline


class Decimal(Float):
    """:class:`Decimal` represents a value that must be valid
    when passed to decimal.Decimal()

    Usage::

        from kim import Mapper
        from kim import field

        class UserMapper(Mapper):
            __type__ = User

            score = field.Decimal(precision=4, min=0, max=1.5)

    """

    opts_class = FloatFieldOpts
    marshal_pipeline = DecimalMarshalPipeline
    serialize_pipeline = DecimalSerializePipeline


class BooleanFieldOpts(FieldOpts):
    """Custom FieldOpts class that provides additional config options for
    :class:`Boolean`.

    """

    def __init__(self, **kwargs):
        """ Construct a new instance of :class:`BooleanFieldOpts`
        and set config options

        :param true_boolean_values: Specify an array of values that will validate as
            being 'true' when the field is marshaled.
        :param false_boolean_values: Specify an array of values that will validate as
            being 'false' when the field is marshaled.

        :raises: :class:`FieldOptsError`
        :returns: None
        """
        self.true_boolean_values = \
            kwargs.pop('true_boolean_values',
                       [True, 'true', '1', 1, 'True'])
        self.false_boolean_values = \
            kwargs.pop('false_boolean_values',
                       [False, 'false', '0', 0, 'False'])

        super(BooleanFieldOpts, self).__init__(**kwargs)
        self.choices = set(self.true_boolean_values +
                           self.false_boolean_values)


class Boolean(Field):
    """:class:`Boolean` represents a value that must be valid
    boolean type.

    Usage::

        from kim import Mapper
        from kim import field

        class UserMapper(Mapper):
            __type__ = User

            active = field.Boolean(
                required=True,
                true_boolean_values=[True, 'true', 1],
                false_boolean_values=[False, 'false', 0])

    """

    opts_class = BooleanFieldOpts
    marshal_pipeline = BooleanMarshalPipeline
    serialize_pipeline = BooleanSerializePipeline


class NestedFieldOpts(FieldOpts):
    """Custom FieldOpts class that provides additional config options for
    :class:`Nested`.

    """

    def __init__(self, mapper_or_mapper_name, **kwargs):
        """Construct a new instance of :class:`NestedFieldOpts`

        :param mapper_or_mapper_name: a required instance of a :class:`Mapper`
            or a valid mapper name
        :param role: specify the name of a role to use on the Nested mapper
        :param collection_class: provide a custom type to be used when
            mapping many nested objects
        :param getter: provide a function taking a pipeline session which returns
            the object to be set on this field, or None if it can't find one.
            This is useful where your API accepts simply `{'id': 2}` but you
            want a full object to be set
        :param allow_updates:  Allow existing objects returned by the ``getter`` function
            to be updated.
        :param allow_updates_in_place: Whereas allow_updates requires the getter to
            return an existing object which it will then update, allow_updates_in_place
            will make updates to any existing object it finds at the specified key.
        :param allow_create: If the ``getter`` returns None, allow the Nested field to
            create a new instance.
        :param allow_partial_updates: Allow existing object to be updated using a subset
            of the fields defined on the Nested field.
        """
        self.mapper = mapper_or_mapper_name
        self.role = kwargs.pop('role', '__default__')
        self.collection_class = kwargs.pop('collection_class', list)
        self.getter = kwargs.pop('getter', None)
        self.allow_updates = kwargs.pop('allow_updates', False)
        self.allow_updates_in_place = kwargs.pop(
            'allow_updates_in_place', False)
        self.allow_partial_updates = kwargs.pop(
            'allow_partial_updates', False)
        self.allow_create = kwargs.pop('allow_create', False)
        super(NestedFieldOpts, self).__init__(**kwargs)


class Nested(Field):
    """:class:`Nested` represents an object that is represented by another
    mapper.

    Usage::

        from kim import Mapper
        from kim import field

        class PostMapper(Mapper):
            __type__ = User

            id = field.String()
            name= field.String()
            content = field.String()
            user = field.Nested(
                'UserMapper',
                role='public',
                getter=user_getter,
                allow_upadtes=False,
                allow_partial_updates=False,
                allow_updates_in_place=False,
                allow_create=False,
                required=True)

    .. seealso::
        :class:`NestedFieldOpts`

    """

    opts_class = NestedFieldOpts
    marshal_pipeline = NestedMarshalPipeline
    serialize_pipeline = NestedSerializePipeline

    def __init__(self, *args, **kwargs):

        super(Nested, self).__init__(*args, **kwargs)
        self._mapper_class = None

    def get_mapper(self, as_class=False, **mapper_params):
        """Retrieve the specified mapper from the Mapper registry.

        :param as_class: Return the Mapper class object without
            calling the constructor.  This is typically used when nested
            is mapping many objects.
        :param mapper_params: A dict of kwarg's to pass to the specified
            mappers constructor

        :rtype: :class:`Mapper`
        :returns: a new instance of the specified mapper
        """

        from .mapper import get_mapper_from_registry

        if self._mapper_class is None:
            mapper = get_mapper_from_registry(self.opts.mapper)
            self._mapper_class = mapper

        if as_class:
            return self._mapper_class
        else:
            return self._mapper_class(**mapper_params)


class CollectionFieldOpts(FieldOpts):
    """Custom FieldOpts class that provides additional config options for
    :class:`Collection`.

    """

    def __init__(self, field, **kwargs):
        """Construct a new instance of :class:`.CollectionFieldOpts`

        :param field: Specify the field type mpapped inside of this collection.  This
            may be any :class:`Field` type.
        :param unique_on: Specify a key that is used to check the collection
            for duplicates.

        """
        self.field = field
        try:
            self.field.name
        except FieldError:
            pass
        else:
            raise FieldError('name/attribute_name/source should '
                             'not be passed to a wrapped field.')

        self.field.opts._is_wrapped = True
        self.unique_on = kwargs.pop('unique_on', None)
        super(CollectionFieldOpts, self).__init__(**kwargs)

    def set_name(self, *args, **kwargs):
        """proxy access to the :class:`FieldOpts` defined for
        this collections field.

        :returns: None

        """
        self.field.opts.set_name(*args, **kwargs)
        super(CollectionFieldOpts, self).set_name(*args, **kwargs)

    def get_name(self):
        """Proxy access to the :class:`FieldOpts` defined for
        this collections field.

        :rtype: str
        :returns: The value of get_name from the collections Field.

        """

        return self.field.name

    def validate(self):
        """Exra validation for Collection Field.

        :raises: FieldOptsError
        """

        if not isinstance(self.field, Field):
            raise FieldOptsError('Collection requires a valid Field '
                                 'instance as its first argument')


class Collection(Field):
    """:class:`Collection` represents collection of other field types,
    typically stored in a list.

    Usage::

        from kim import Mapper
        from kim import field

        class UserMapper(Mapper):
            __type__ = User

            id = field.String()
            friends = field.Collection(field.Nested('UserMapper', required=True))
            user_ids = field.Collection(field.String())

    .. seealso::
        :class:`CollectionFieldOpts`

    """

    marshal_pipeline = CollectionMarshalPipeline
    serialize_pipeline = CollectionSerializePipeline
    opts_class = CollectionFieldOpts


class StaticFieldOpts(FieldOpts):
    """Custom FieldOpts class that provides additional config options for
    :class:`Static`.

    """

    def __init__(self, value, **kwargs):
        """Construct a new instance of :class:`StaticFieldOpts`

        :param value: specify the static value to return when this field
            is serialized.

        """
        self.value = value
        super(StaticFieldOpts, self).__init__(**kwargs)
        self.read_only = True


class Static(Field):
    """:class:`Static` represents a field that outputs a constant value.

    This field is implicitly read_only and therefore is typically only used
    during serialization flows.

    Usage::

        from kim import Mapper
        from kim import field

        class UserMapper(Mapper):
            __type__ = User

            id = field.String()
            object_type = field.Static(value='user')
    """
    opts_class = StaticFieldOpts
    serialize_pipeline = StaticSerializePipeline


class DateTimeFieldOpts(FieldOpts):
    """Custom FieldOpts class that provides additional config options for
    :class:`DateTime`.

    """

    def __init__(self, **kwargs):
        """Construct a new instance of :class:`DateTimeFieldOpts`

        :param format_str: Specify a format string used to validate an
            incoming date string.  If not value is passed then the format defaults to
            iso8601.

        """
        self.date_format = kwargs.pop('format_str', 'iso8601')
        super(DateTimeFieldOpts, self).__init__(**kwargs)
        self.error_msgs['invalid'] = 'Not a valid datetime.'


class DateFieldOpts(FieldOpts):
    """Custom FieldOpts class that provides additional config options for
    :class:`Date`.

    """

    def __init__(self, **kwargs):
        """Construct a new instance of :class:`Date`

        :param format_str: Specify a format string used to validate an
            incoming date string.  If not value is passed then the format defaults to
            iso8601.

        """
        self.date_format = kwargs.pop('format_str', '%Y-%m-%d')
        super(DateFieldOpts, self).__init__(**kwargs)
        self.error_msgs['invalid'] = 'Not a valid date.'


class DateTime(Field):
    """:class:`DateTime` represents an iso8601 encoded date time

    .. code-block:: python

        from kim import Mapper
        from kim import field

        class UserMapper(Mapper):
            __type__ = User

            created_at = field.DateTime(required=True)

    """

    opts_class = DateTimeFieldOpts
    marshal_pipeline = DateTimeMarshalPipeline
    serialize_pipeline = DateTimeSerializePipeline


class Date(DateTime):
    """:class:`Date` represents a date object

    .. code-block:: python

        from kim import Mapper
        from kim import field

        class UserMapper(Mapper):
            __type__ = User

            signup_date = field.Date(required=True)

    """

    opts_class = DateFieldOpts
    marshal_pipeline = DateMarshalPipeline



import pytest

from ..conftest import get_mapper_session
from kim.field import FieldInvalid, Boolean
from kim.pipelines.base import Session, is_valid_choice


def test_is_allowed_value():

    field = Boolean(name='test')
    session = Session(field, 'test', {})

    with pytest.raises(FieldInvalid):
        is_valid_choice(session)

    session.data = True
    assert is_valid_choice(session) is True

    session.data = 'true'
    assert is_valid_choice(session) == 'true'

    session.data = '1'
    assert is_valid_choice(session) == '1'

    session.data = 'True'
    assert is_valid_choice(session) == 'True'

    session.data = False
    assert is_valid_choice(session) is False

    session.data = 'false'
    assert is_valid_choice(session) == 'false'

    session.data = '0'
    assert is_valid_choice(session) == '0'

    session.data = 0
    assert is_valid_choice(session) == 0

    session.data = 'False'
    assert is_valid_choice(session) == 'False'


def test_is_allowed_value_with_custom_values():

    field = Boolean(name='test', true_boolean_values=['foo'],
                    false_boolean_values=['bar'])

    data = True
    session = Session(field, data, {})

    with pytest.raises(FieldInvalid):
        is_valid_choice(session)

    session.data = False
    with pytest.raises(FieldInvalid):
        is_valid_choice(session)

    session.data = 'foo'
    assert is_valid_choice(session) == 'foo'
    session.data = 'bar'
    assert is_valid_choice(session) == 'bar'


def test_boolean_input():

    field = Boolean(name='is_active', required=True)

    output = {}
    mapper_session = get_mapper_session(
        data={'is_active': False, 'email': 'mike@mike.com'}, output=output)

    field.marshal(mapper_session)
    assert output == {'is_active': False}

    mapper_session = get_mapper_session(
        data={'is_active': 'false', 'email': 'mike@mike.com'}, output=output)

    field.marshal(mapper_session)
    assert output == {'is_active': False}

    mapper_session = get_mapper_session(
        data={'is_active': True, 'email': 'mike@mike.com'}, output=output)

    field.marshal(mapper_session)
    assert output == {'is_active': True}

    mapper_session = get_mapper_session(
        data={'is_active': 'true', 'email': 'mike@mike.com'}, output=output)

    field.marshal(mapper_session)
    assert output == {'is_active': True}


def test_boolean_input_with_allow_none():

    field = Boolean(name='is_active', required=False, allow_none=True)

    output = {}
    mapper_session = get_mapper_session(
        data={'is_active': None, 'email': 'mike@mike.com'}, output=output)

    field.marshal(mapper_session)
    assert output == {'is_active': None}


def test_boolean_output():

    class Foo(object):
        is_active = True

    field = Boolean(name='is_active', required=True)

    output = {}
    mapper_session = get_mapper_session(obj=Foo(), output=output)
    field.serialize(mapper_session)
    assert output == {'is_active': True}



import pytest

from kim.mapper import Mapper, MapperError
from kim.field import FieldInvalid
from kim import field
from kim.pipelines import marshaling

from ..conftest import get_mapper_session
from ..helpers import TestType


def test_nested_defers_mapper_checks():
    """ensure that instantiating a nested field with an invalid mapper
    doesn't emit an error until the nested mapper is actually needed.

    """
    field.Nested('IDontExist', name='user')


def test_nested_get_mapper_str_mapper_name():
    class UserMapper(Mapper):

        __type__ = dict

        id = field.String(required=True, read_only=True)
        name = field.String()

    f = field.Nested('UserMapper', name='user')
    assert isinstance(f.get_mapper(data={'foo': 'id'}), UserMapper)


def test_get_mapper_mapper_type():

    class UserMapper(Mapper):

        __type__ = dict

        id = field.String(required=True, read_only=True)
        name = field.String()

    f = field.Nested(UserMapper, name='user')
    assert isinstance(f.get_mapper(data={'foo': 'id'}), UserMapper)


def test_get_mapper_not_registered():

    f = field.Nested('UserMapper', name='user')
    with pytest.raises(MapperError):
        f.get_mapper(data={'foo': 'id'})


def test_get_mapper_not_a_valid_mapper():

    class Foo(object):
        pass

    f = field.Nested(Foo, name='user')
    with pytest.raises(MapperError):
        f.get_mapper(data={'foo': 'id'})


def test_marshal_nested():

    class UserMapper(Mapper):

        __type__ = dict

        id = field.String(required=True)
        name = field.String()

    data = {'id': 2, 'name': 'bob', 'user': {'id': '1', 'name': 'mike'}}
    test_field = field.Nested('UserMapper', name='user', allow_create=True)

    output = {}
    mapper_session = get_mapper_session(data=data, output=output)
    test_field.marshal(mapper_session)
    assert output == {'user': {'id': '1', 'name': 'mike'}}


def test_serialise_nested():

    class UserMapper(Mapper):

        __type__ = dict

        id = field.String(required=True, read_only=True)
        name = field.String()

    data = {'id': 2, 'name': 'bob', 'user': {'id': '1', 'name': 'mike'}}
    test_field = field.Nested('UserMapper', name='user')

    output = {}
    mapper_session = get_mapper_session(obj=data, output=output)
    test_field.serialize(mapper_session)
    assert output == {'user': {'id': '1', 'name': 'mike'}}


def test_serialise_nested_with_role():

    class UserMapper(Mapper):

        __type__ = dict

        id = field.String(required=True, read_only=True)
        name = field.String()

        __roles__ = {
            'public': ['name', ]
        }

    data = {'id': 2, 'name': 'bob', 'user': {'id': '1', 'name': 'mike'}}
    test_field = field.Nested('UserMapper', name='user', role='public')

    output = {}
    mapper_session = get_mapper_session(obj=data, output=output)
    test_field.serialize(mapper_session)
    assert output == {'user': {'name': 'mike'}}


def test_marshal_nested_with_role():

    class UserMapper(Mapper):

        __type__ = dict

        id = field.String(required=True, read_only=True)
        name = field.String()

        __roles__ = {
            'public': ['name', ]
        }

    data = {'id': 2, 'name': 'bob', 'user': {'id': '1', 'name': 'mike'}}
    test_field = field.Nested('UserMapper', name='user', role='public',
                              allow_create=True)

    output = {}
    mapper_session = get_mapper_session(data=data, output=output)
    test_field.marshal(mapper_session)
    assert output == {'user': {'name': 'mike'}}


def test_marshal_nested_with_read_only_field():

    class UserMapper(Mapper):

        __type__ = dict

        id = field.String(required=True, read_only=True)
        name = field.String()

    data = {'id': 2, 'name': 'bob', 'user': {'id': '1', 'name': 'mike'}}
    test_field = field.Nested('UserMapper', name='user', allow_create=True)

    output = {}
    mapper_session = get_mapper_session(data=data, output=output)
    test_field.marshal(mapper_session)
    assert output == {'user': {'name': 'mike'}}


def test_marshal_read_only_nested_mapper():

    class UserMapper(Mapper):

        __type__ = dict

        id = field.String(required=True, read_only=True)
        name = field.String()

    data = {'id': 2, 'name': 'bob', 'user': {'id': '1', 'name': 'mike'}}
    test_field = field.Nested('UserMapper', name='user', read_only=True)

    output = {}
    mapper_session = get_mapper_session(data=data, output=output)
    test_field.marshal(mapper_session)
    assert output == {}


def test_marshal_nested_with_getter():

    class UserMapper(Mapper):

        __type__ = dict

        id = field.String(required=True)
        name = field.String()

    users = {
        '1': {'id': '1', 'name': 'mike'},
        '2': {'id': '2', 'name': 'jack'}
    }

    def getter(session):
        return users[session.data['id']]

    test_field = field.Nested('UserMapper', name='user', getter=getter)

    data1 = {'id': 2, 'name': 'bob', 'user': {'id': '1'}}
    output = {}
    mapper_session = get_mapper_session(data=data1, output=output)
    test_field.marshal(mapper_session)
    assert output == {'user': {'id': '1', 'name': 'mike'}}

    data2 = {'id': 2, 'name': 'bob', 'user': {'id': '2'}}
    output = {}
    mapper_session = get_mapper_session(data=data2, output=output)
    test_field.marshal(mapper_session)
    assert output == {'user': {'id': '2', 'name': 'jack'}}


def test_marshal_nested_with_getter_failure():

    class UserMapper(Mapper):

        __type__ = dict

        id = field.String(required=True)
        name = field.String()

    def getter(session):
        return None

    test_field = field.Nested('UserMapper', name='user', getter=getter)

    data = {'id': 2, 'name': 'bob', 'user': {'id': '1'}}
    output = {}
    mapper_session = get_mapper_session(data=data, output=output)
    with pytest.raises(FieldInvalid):
        test_field.marshal(mapper_session)


def test_marshal_nested_with_defaults():
    # Users may only be passed by id and may not be updated

    class UserMapper(Mapper):

        __type__ = dict

        id = field.String(required=True)
        name = field.String()

    user = {'id': '1', 'name': 'mike'}

    def getter(session):
        return user

    test_field = field.Nested('UserMapper', name='user', getter=getter)

    data1 = {'id': 2, 'name': 'bob', 'user': {
        'id': '1', 'name': 'this should be ignored'}}
    output = {}
    mapper_session = get_mapper_session(data=data1, output=output)
    test_field.marshal(mapper_session)
    assert output == {'user': {'id': '1', 'name': 'mike'}}

    assert user['name'] == 'mike'


def test_marshal_nested_with_allow_updates():
    # Users may only be passed by id and may be updated, but not created

    class UserMapper(Mapper):

        __type__ = dict

        id = field.String(required=True)
        name = field.String()

    user = {'id': '1', 'name': 'mike'}

    def getter(session):
        if session.data['id'] == '1':
            return user

    test_field = field.Nested('UserMapper', name='user', getter=getter,
                              allow_updates=True)

    data1 = {'id': 2, 'name': 'bob', 'user': {
        'id': '1', 'name': 'a new name'}}
    output = {}
    mapper_session = get_mapper_session(data=data1, output=output)
    test_field.marshal(mapper_session)
    assert output == {'user': {'id': '1', 'name': 'a new name'}}
    assert user['name'] == 'a new name'

    data2 = {'id': 2, 'name': 'bob', 'user': {
        'id': '2', 'name': 'should not allow this to be created'}}
    output = {}
    mapper_session = get_mapper_session(data=data2, output=output)
    with pytest.raises(FieldInvalid):
        test_field.marshal(mapper_session)


def test_marshal_nested_with_allow_create_only():
    # Users may only be passed by id or created if they don't exist

    class UserMapper(Mapper):

        __type__ = dict

        id = field.String(required=True)
        name = field.String()

    user = {'id': '1', 'name': 'mike'}

    def getter(session):
        if session.data['id'] == '1':
            return user

    test_field = field.Nested('UserMapper', name='user', getter=getter,
                              allow_create=True)

    data1 = {'id': 2, 'name': 'bob', 'user': {
        'id': '1', 'name': 'this should be ignored'}}
    output = {}
    mapper_session = get_mapper_session(data=data1, output=output)
    test_field.marshal(mapper_session)
    assert output == {'user': {'id': '1', 'name': 'mike'}}

    assert user['name'] == 'mike'

    data2 = {'id': 2, 'name': 'bob', 'user': {
        'id': '2', 'name': 'jack'}}
    output = {}
    mapper_session = get_mapper_session(data=data2, output=output)
    test_field.marshal(mapper_session)
    assert output == {'user': {'id': '2', 'name': 'jack'}}


def test_marshal_nested_with_allow_create_and_allow_updates():
    # Users may only be passed by id or created if they don't exist and
    # updated

    class UserMapper(Mapper):

        __type__ = dict

        id = field.String(required=True)
        name = field.String()

    user = {'id': '1', 'name': 'mike'}

    def getter(session):
        if session.data['id'] == '1':
            return user

    test_field = field.Nested('UserMapper', name='user', getter=getter,
                              allow_create=True, allow_updates=True)

    data1 = {'id': 2, 'name': 'bob', 'user': {
        'id': '1', 'name': 'a new name'}}
    output = {}
    mapper_session = get_mapper_session(data=data1, output=output)
    test_field.marshal(mapper_session)
    assert output == {'user': {'id': '1', 'name': 'a new name'}}
    assert user['name'] == 'a new name'

    data2 = {'id': 2, 'name': 'bob', 'user': {
        'id': '2', 'name': 'jack'}}
    output = {}
    mapper_session = get_mapper_session(data=data2, output=output)
    test_field.marshal(mapper_session)
    assert output == {'user': {'id': '2', 'name': 'jack'}}


def test_marshal_nested_with_allow_updates_in_place():
    # No getter required, but the existing user object can be changed

    class UserMapper(Mapper):

        __type__ = dict

        name = field.String()

    user = {'id': '1', 'name': 'mike'}
    output = {'user': user}

    test_field = field.Nested('UserMapper', name='user',
                              allow_updates_in_place=True)

    data1 = {'id': 2, 'name': 'bob', 'user': {'name': 'a new name'}}
    mapper_session = get_mapper_session(data=data1, output=output)
    test_field.marshal(mapper_session)
    assert output == {'user': {'id': '1', 'name': 'a new name'}}
    assert user['name'] == 'a new name'


def test_marshal_nested_partial():

    class UserMapper(Mapper):

        __type__ = dict

        id = field.String(required=True)
        name = field.String()

    class DocumentMapper(Mapper):
        __type__ = dict

        id = field.String(required=True)
        name = field.String()
        user = field.Nested('UserMapper', name='user',
                            allow_partial_updates=True)

    obj = {'id': 2, 'name': 'my document', 'user': {
        'id': '1', 'name': 'existing name'}}

    data = {'name': 'new name', 'user': {'name': 'new user name'}}

    mapper = DocumentMapper(obj=obj, data=data, partial=True)

    mapper.marshal()

    assert obj['name'] == 'new name'
    assert obj['user']['name'] == 'new user name'


def test_self_nesting_marshal():
    class Inner(Mapper):
        __type__ = dict

        name = field.String(source='user_name')

    class Outer(Mapper):
        __type__ = dict

        user = field.Nested(Inner, source='__self__', allow_create=True)
        status = field.Integer()

    data = {'user': {'name': 'jack'}, 'status': 200}

    result = Outer(data=data).marshal()

    assert result == {'user_name': 'jack', 'status': 200}



import pytest

from kim import Mapper, field
from kim.exception import MappingInvalid
from kim.field import FieldInvalid

from ..conftest import get_mapper_session
from ..helpers import TestType


def test_collection_proxies_name_to_wrapped_field():

    f = field.Collection(field.Integer(), name='post_ids')
    f2 = field.Collection(field.String(), name='test')

    with pytest.raises(field.FieldError):
        field.Collection(field.String(name='foo'))

    class UserMapper(Mapper):

        __type__ = TestType

        id = field.String(required=True)
        tags = field.Collection(field.String())

    assert f.name == 'post_ids'
    assert f2.name == 'test'

    mapper = UserMapper({})
    assert mapper.fields['tags'].name == 'tags'


def test_marshal_collection_requires_list():

    f = field.Collection(field.Integer(), name='post_ids')
    output = {}

    mapper_session = get_mapper_session(data={'post_ids': 1}, output=output)
    with pytest.raises(field.FieldInvalid):
        f.marshal(mapper_session)


def test_marshal_flat_collection():

    f = field.Collection(field.Integer(), name='post_ids', source='posts')
    output = {}
    data = {
        'post_ids': [2, 1]
    }
    mapper_session = get_mapper_session(data=data, output=output)
    f.marshal(mapper_session)
    assert output == {'posts': [2, 1]}


def test_serialize_flat_collection():

    f = field.Collection(field.Integer(), name='post_ids', source='posts')
    output = {}
    data = {
        'posts': [2, 1]
    }
    mapper_session = get_mapper_session(obj=data, output=output)
    f.serialize(mapper_session)
    assert output == {'post_ids': [2, 1]}


def test_marshal_read_only_collection():

    f = field.Collection(field.Integer(), name='post_ids', read_only=True)
    output = {}
    data = {
        'post_ids': [2, 1]
    }
    mapper_session = get_mapper_session(data=data, output=output)
    f.marshal(mapper_session)
    assert output == {}


def test_marshal_nested_collection_allow_create():

    class UserMapper(Mapper):

        __type__ = TestType

        id = field.String(required=True)
        name = field.String()
    data = {'id': 2, 'name': 'bob', 'users': [{'id': '1', 'name': 'mike'}]}

    f = field.Collection(field.Nested('UserMapper', allow_create=True),
                         name='users')
    output = {}
    mapper_session = get_mapper_session(data=data, output=output)
    f.marshal(mapper_session)
    assert output == {'users': [TestType(id='1', name='mike')]}


def test_marshal_nested_collection_default():

    class UserMapper(Mapper):

        __type__ = TestType

        id = field.String(required=True)
        name = field.String()

    user = TestType(id='1', name='mike')
    data = {'id': 2, 'name': 'bob', 'users': [{'id': '1',
                                              'name': 'ignore this'}]}

    def getter(session):
        if session.data['id'] == '1':
            return user

    f = field.Collection(field.Nested('UserMapper', getter=getter),
                         name='users')
    output = {}
    mapper_session = get_mapper_session(data=data, output=output)
    f.marshal(mapper_session)
    assert output == {'users': [user]}
    assert user.name == 'mike'


def test_marshal_nested_collection_allow_updates():

    class UserMapper(Mapper):

        __type__ = TestType

        id = field.String(required=True)
        name = field.String()

    user = TestType(id='1', name='mike')
    data = {'id': 2, 'name': 'bob', 'users': [{'id': '1', 'name': 'new name'}]}

    def getter(session):
        if session.data['id'] == '1':
            return user

    f = field.Collection(field.Nested('UserMapper', getter=getter,
                         allow_updates=True), name='users')
    output = {}
    mapper_session = get_mapper_session(data=data, output=output)
    f.marshal(mapper_session)
    assert output == {'users': [user]}
    assert user.name == 'new name'


def test_marshal_nested_collection_allow_updates_in_place():

    class UserMapper(Mapper):

        __type__ = TestType

        name = field.String()

    user = TestType(id='1', name='mike')
    data = {'id': 2, 'name': 'bob', 'users': [{'name': 'new name'}]}

    f = field.Collection(field.Nested('UserMapper',
                         allow_updates_in_place=True), name='users')
    output = {'users': [user]}
    mapper_session = get_mapper_session(data=data, output=output)
    f.marshal(mapper_session)
    assert output == {'users': [user]}
    assert user.name == 'new name'


def test_marshal_nested_collection_allow_updates_in_place_too_many():
    # We're updating in place, but there are more users in the input data
    # than already exist so an error should be raised

    class UserMapper(Mapper):

        __type__ = TestType

        name = field.String()

    user = TestType(id='1', name='mike')
    data = {'id': 2, 'name': 'bob', 'users': [
        {'name': 'name1'}, {'name': 'name2'}]}

    f = field.Collection(field.Nested('UserMapper',
                         allow_updates_in_place=True), name='users')
    output = {'users': [user]}
    mapper_session = get_mapper_session(data=data, output=output)
    with pytest.raises(FieldInvalid):
        f.marshal(mapper_session)


def test_marshal_nested_collection_allow_updates_in_place_too_many_with_allow_create():
    # We're updating in place, there are more users in the input data
    # but allow_create is also enabled, so a new user should be added

    class UserMapper(Mapper):

        __type__ = TestType

        name = field.String()

    user = TestType(id='1', name='mike')
    data = {'id': 2, 'name': 'bob', 'users': [
        {'name': 'name1'}, {'name': 'name2'}]}

    f = field.Collection(field.Nested('UserMapper',
                         allow_updates_in_place=True, allow_create=True),
                         name='users')
    output = {'users': [user]}
    mapper_session = get_mapper_session(data=data, output=output)

    f.marshal(mapper_session)

    assert output == {
        'users': [TestType(id='1', name='name1'), TestType(name='name2')]}


def test_serialize_nested_collection():

    class UserMapper(Mapper):

        __type__ = TestType

        id = field.String(required=True, read_only=True)
        name = field.String()

    users = [TestType(id='1', name='mike'), TestType(id='2', name='jack')]
    post = TestType(id='1', users=users)

    output = {}
    f = field.Collection(field.Nested('UserMapper'), name='users')
    mapper_session = get_mapper_session(obj=post, output=output)
    f.serialize(mapper_session)

    assert output == {'users': [{'id': '1', 'name': 'mike'},
                                {'id': '2', 'name': 'jack'}]}


def test_marshal_collection_sets_parent_session_scope():

    class UserMapper(Mapper):

        __type__ = TestType

        id = field.String(required=True, read_only=True)
        name = field.String()

    def assert_scope(session):

        assert session.parent is not None
        assert isinstance(session.parent.output, TestType)

        return TestType(id=session.data['id'], name='foo')

    class PostMapper(Mapper):

        __type__ = TestType

        readers = field.Collection(field.Nested(UserMapper, getter=assert_scope))

    data = {'id': '1', 'readers': [{'id': '1', 'name': 'mike'}]}

    mapper = PostMapper(data=data)
    mapper.marshal()


def test_marshal_collection_inherits_parent_session_partial():

    class UserMapper(Mapper):

        __type__ = TestType

        id = field.String(required=True, read_only=True)
        name = field.String()

    def assert_scope(session):

        assert session.mapper_session.partial is True

        return TestType(id=session.data['id'], name='foo')

    class PostMapper(Mapper):

        __type__ = TestType

        readers = field.Collection(field.Nested(UserMapper, getter=assert_scope))

    data = {'id': '1', 'readers': [{'id': '1', 'name': 'mike'}]}

    mapper = PostMapper(data=data, partial=True)
    mapper.marshal()


def test_serialize_collection_sets_parent_session_scope():

    class UserMapper(Mapper):

        __type__ = TestType

        id = field.String(required=True, read_only=True)
        name = field.String()

    def assert_scope(session):

        assert session.parent is not None
        assert isinstance(session.parent.output, TestType)

        return TestType(id=session.data['id'], name='foo')

    class PostMapper(Mapper):

        __type__ = TestType

        readers = field.Collection(field.Nested(UserMapper, getter=assert_scope))

    users = [TestType(id='1', name='mike'), TestType(id='2', name='jack')]
    post = TestType(id='1', users=users)

    mapper = PostMapper(obj=post)
    mapper.serialize()


def test_marshal_collection_unique_on():

    class UserMapper(Mapper):

        __type__ = TestType

        id = field.String(required=True)
        name = field.String()

    class PostMapper(Mapper):

        __type__ = TestType

        readers = field.Collection(
            field.Nested(UserMapper, allow_create=True),
            unique_on='id')

    data = {
        'readers': [{'id': '1', 'name': 'jack'}, {'id': '1', 'name': 'jack'}]
    }

    mapper = PostMapper(data=data)

    with pytest.raises(MappingInvalid):
        mapper.marshal()

    # Check error not raised with valid data
    data = {
        'readers': [{'id': '1', 'name': 'jack'}, {'id': '2', 'name': 'mike'}]
    }

    mapper = PostMapper(data=data)
    mapper.marshal()


def test_marshal_collection_required_false():

    class UserMapper(Mapper):

        __type__ = TestType

        id = field.String(required=True)
        name = field.String()

    class PostMapper(Mapper):

        __type__ = TestType

        readers = field.Collection(
            field.Nested(UserMapper, allow_create=True),
            required=False)

    data = {}

    mapper = PostMapper(data=data)

    output = mapper.marshal()

    assert output.readers == []



# kim/exception.py
# Copyright (C) 2014-2015 the Kim authors and contributors
# <see AUTHORS file>
#
# This module is part of Kim and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php


class KimException(Exception):
    """Base Exception for all Kim exception types."""

    def __init__(self, message, *args, **kwargs):

        self.message = message
        super(KimException, self).__init__(message, *args, **kwargs)


class MapperError(KimException):
    """MapperError is raised from a mapper that was unable to instantiate correctly."""
    pass


class MappingInvalid(KimException):

    def __init__(self, errors, *args, **kwargs):
        self.errors = errors
        super(MappingInvalid, self).__init__('Mapping invalid', *args, **kwargs)


class RoleError(KimException):
    pass


class FieldOptsError(KimException):
    pass


class FieldError(KimException):
    pass


class FieldInvalid(KimException):
    def __init__(self, *args, **kwargs):
        self.field = kwargs.pop('field')
        super(FieldInvalid, self).__init__(*args, **kwargs)


class StopPipelineExecution(KimException):
    pass



# kim/util.py
# Copyright (C) 2014-2015 the Kim authors and contributors
# <see AUTHORS file>
#
# This module is part of Kim and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from datetime import datetime  # NOQA

from collections import defaultdict


_creation_order = 1


def set_creation_order(instance):
    """Assign a '_creation_order' sequence to the given instance.
    This allows multiple instances to be sorted in order of creation
    (typically within a single thread; the counter is not particularly
    threadsafe).
    """
    global _creation_order
    instance._creation_order = _creation_order
    _creation_order += 1


def _attr_or_key(obj, name, _isinstance=isinstance, _dict=dict, getter=getattr):
    """Attempt to return the attr stored against the specified name from a dict or
    from an attribute on an object.

    :param obj: The obj to request the attribute from
    :param name: The name of the attribute or key on the obj
    :param _isinstance: isinstance built-in
    :param _dict: isinstance built-in
    :param _getattr: getattr built-in

    Note
    ~~~~~

    Just a brief word on the odd re-assignment of built in functions to kwargs.
    Given the huge number of times this function is called in the benchmarking suite,
    Python having to call the builtins from the global scope is actually fairly slow.

    By assigning them to kwargs they will be compiled in the module's scope and will
    be found using the LOAD_FAST lookup.  You can read more about this technique here
    http://jamesls.com/micro-optimizations-in-python-code-speeding-up-lookups.html

    This technique yielded between 3-5% performance increase so we felt it was worth the
    trade of with it being pretty weird.

    .. version-added: 1.1.0
    """
    if _isinstance(obj, _dict):
        return obj.get(name)
    else:
        return getter(obj, name, None)


def _set_attr_or_key(obj, name, value, _isinstance=isinstance, _dict=dict, setter=setattr):
    """Attempt to set the attr stored against the specified name on a dict or
    on an attribute on an object.

    :param obj: The obj to request the attribute from
    :param name: The name of the attribute or key on the obj
    :param value: The name value being set
    :param _isinstance: isinstance built-in
    :param _dict: isinstance built-in
    :param setter: setattr built-in

    Note
    ~~~~~

    Just a brief word on the odd re-assignment of built in functions to kwargs.
    Given the huge number of times this function is called in the benchmarking suite,
    Python having to call the builtins from the global scope is actually fairly slow.

    By assigning them to kwargs they will be compiled in the module's scope and will
    be found using the LOAD_FAST lookup.  You can read more about this technique here
    http://jamesls.com/micro-optimizations-in-python-code-speeding-up-lookups.html

    This technique yielded between 3-5% performance increase so we felt it was worth the
    trade of with it being pretty weird.

    .. version-added: 1.1.0
    """
    if _isinstance(obj, _dict):
        obj[name] = value
    else:
        setter(obj, name, value)


def attr_or_key(obj, name):
    """attempt to use getattr to access an attribute of obj, if that fails
    assume obj support key based look ups like a dict.

    Supports dot syntax to span nested objects/dicts eg 'foo.bar.baz'
    """
    components = name.split('.')
    for component in components:
        obj = _attr_or_key(obj, component)
    return obj


def set_attr_or_key(obj, name, value):
    """attempt to use getattr to access an attribute of obj, if that fails
    assume obj support key based look ups like a dict.

    Supports dot syntax to span nested objects/dicts eg 'foo.bar.baz'
    """
    components = name.split('.')
    for component in components[:-1]:
        obj = _attr_or_key(obj, component)
    _set_attr_or_key(obj, components[-1], value)


def attr_or_key_update(obj, value):
    """If obj is a dict, add keys from value to it with update(),
    otherwise use setattr to set every attribute from value on obj
    """
    if obj is not None and value is not None:
        if isinstance(obj, dict):
            obj.update(value)
        else:
            for k, v in value.items():
                setattr(obj, k, v)


def recursive_defaultdict():
    """A simple recurrsive version of ``collections.defaultdict``

    """
    return defaultdict(recursive_defaultdict)



# kim/pipelines/boolean.py
# Copyright (C) 2014-2015 the Kim authors and contributors
# <see AUTHORS file>
#
# This module is part of Kim and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from .base import pipe, is_valid_choice
from .marshaling import MarshalPipeline
from .serialization import SerializePipeline


@pipe()
def coerce_to_boolean(session):
    """Given a valid boolean value, ie True, 'true', 'false', False, 0, 1
    set the data to the python boolean type True or False

    :param session: Kim pipeline session instance
    """
    if session.data in session.field.opts.true_boolean_values:
        session.data = True
    else:
        session.data = False

    return session.data


class BooleanMarshalPipeline(MarshalPipeline):
    """BooleanMarshalPipeline

    .. seealso::
        :func:`kim.pipelines.base.is_valid_choice`
        :func:`kim.pipelines.boolean.coerce_to_boolean`
        :class:`kim.pipelines.marshaling.MarshalPipeline`
    """

    validation_pipes = [is_valid_choice, ] + MarshalPipeline.validation_pipes
    process_pipes = [coerce_to_boolean, ] + MarshalPipeline.process_pipes


class BooleanSerializePipeline(SerializePipeline):
    """BooleanSerializePipeline

    .. seealso::
        :class:`kim.pipelines.serialization.SerializePipeline`
    """
    pass



# kim/pipelines/string.py
# Copyright (C) 2014-2015 the Kim authors and contributors
# <see AUTHORS file>
#
# This module is part of Kim and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

from kim.exception import FieldInvalid
from kim.utils import attr_or_key

from .base import pipe
from .marshaling import MarshalPipeline
from .serialization import SerializePipeline


@pipe(run_if_none=True)
def marshall_collection(session):
    """iterate over each item in ``data`` and marshal the item through the
    wrapped field defined for this collection

    :param session: Kim pipeline session instance

    TODO(mike) this should be called marshal_collection
    """
    wrapped_field = session.field.opts.field
    existing_value = attr_or_key(session.output, session.field.opts.source)

    output = []

    if session.data is not None:
        if not hasattr(session.data, '__iter__'):
            raise session.field.invalid('type_error')

        for i, datum in enumerate(session.data):
            _output = {}
            # If the object already exists, try to match up the existing elements
            # with those in the input json
            if existing_value is not None:
                try:
                    _output[wrapped_field.opts.source] = existing_value[i]
                except IndexError:
                    pass

            mapper_session = session.mapper.get_mapper_session(datum, _output)
            wrapped_field.marshal(mapper_session, parent_session=session)

            result = _output[wrapped_field.opts.source]
            output.append(result)

    session.data = output
    return session.data


@pipe()
def serialize_collection(session):
    """iterate over each item in ``data`` and serialize the item through the
    wrapped field defined for this collection

    :param session: Kim pipeline session instance

    """
    wrapped_field = session.field.opts.field
    field_name = wrapped_field.name
    output = []

    mapper_session = session.mapper.get_mapper_session(None, {})

    # If the wrapped field uses a mapper, fetch it once to avoid looking up the mapper
    # from the registry for each item in the collection.
    session.nested_mapper = getattr(
        wrapped_field,
        'get_mapper',
        lambda **kwargs: None)(as_class=True)

    for datum in session.data:
        mapper_session.data = datum
        mapper_session.output = {}
        wrapped_field.serialize(mapper_session, parent_session=session)
        output.append(mapper_session.output[field_name])

    session.data = output
    return session.data


@pipe()
def check_duplicates(session):
    """iterate over collection and check for duplicates if th unique_on FieldOpt has been
    set of this Collection field

    TODO(mike) This should only run if the wrapped field is a nested collection

    """
    data = session.data
    key = session.field.opts.unique_on
    if key:
        keys = [attr_or_key(a, key) for a in data]
        if len(keys) != len(set(keys)):
            raise session.field.invalid(error_type='duplicates')
    return data


class CollectionMarshalPipeline(MarshalPipeline):
    """CollectionMarshalPipeline

    .. seealso::
        :func:`kim.pipelines.collection.check_duplicates`
        :func:`kim.pipelines.collection.marshal_collection`
        :class:`kim.pipelines.marshaling.MarshalPipeline`
    """

    input_pipes = MarshalPipeline.input_pipes + [check_duplicates, marshall_collection]


class CollectionSerializePipeline(SerializePipeline):
    """CollectionSerializePipeline

    .. seealso::
        :func:`kim.pipelines.collection.serialize_collection`
        :class:`kim.pipelines.serialization.SerializePipeline`
    """

    process_pipes = [serialize_collection, ] + SerializePipeline.process_pipes



# kim/mapper.py
# Copyright (C) 2014-2016 the Kim authors and contributors
# <see AUTHORS file>
#
# This module is part of Kim and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php

import warnings
import weakref
import six
import inspect

from collections import OrderedDict, defaultdict

from .exception import MapperError, MappingInvalid
from .field import Field, FieldError, FieldInvalid
from .role import whitelist, blacklist, Role
from .utils import recursive_defaultdict, attr_or_key
from .pipelines.base import pipe


def mapper_is_defined(mapper_name):

    return mapper_name in _MapperConfig.MAPPER_REGISTRY


def get_mapper_from_registry(mapper_or_name):
    """Search for a defined mapper by name inside of the mapper registry.

    User may pass either a mapper class object or the name of a defined mapper
    as a str

    :param mapper_or_name: a mapper class or name of a mapper
    :raises: :class:`MapperError`
    :return: :class:`Mapper <Mapper>` class
    :rtype: :class:`Mapper`

    Usage::
        >>>mapper = get_mapper_from_registry('UserMapper')
        >>>mapper
        UserMapper

        mapper = get_mapper_from_registry(UserMapper)
        >>>mapper
        UserMapper
    """

    from .mapper import Mapper, mapper_is_defined, _MapperConfig

    if inspect.isclass(mapper_or_name) and issubclass(mapper_or_name, Mapper):
        name = mapper_or_name.__name__
    else:
        name = mapper_or_name

    if not mapper_is_defined(name):
        raise MapperError('%s is not a valid Mapper. '
                          'Is this Mapper defined?'
                          % mapper_or_name)

    reg = _MapperConfig.MAPPER_REGISTRY
    return reg[name]


def add_class_to_registry(classname, cls):
    """Register ``cls`` inside if the registry using ``classname``.  If a cls
    for this name already exists inside the registry an error will be raised.

    :param classname: the name of the class used as the key inside the registry
    :param cls: the class being stored
    :raises :class:`MapperError`
    :return: None
    :rtype: None
    """

    if classname in _MapperConfig.MAPPER_REGISTRY:
        msg = '%s is already a registered Mapper' % classname
        raise MapperError(msg)
    else:
        _MapperConfig.MAPPER_REGISTRY[classname] = cls


# TODO(mike) __docs__
class _MapperConfig(object):

    MAPPER_REGISTRY = weakref.WeakValueDictionary()

    @classmethod
    def setup_mapping(cls, cls_, classname, dict_):
        cfg_cls = _MapperConfig
        cfg_cls(cls_, classname, dict_)

    def __init__(self, cls_, classname, dict_):

        self.dict = dict_
        self.cls = cls_

        for base in reversed(self.cls.__mro__):
            self._extract_defined_pipes(base)
            self._extract_fields(base)
            self._extract_roles(base)

        # If a __default__ role is found in the cls.__roles__ property, assume
        # the user is looking to override the default role and dont create one
        # here.
        if '__default__' not in self.cls.__roles__:
            self.cls.roles['__default__'] = \
                whitelist(*self.cls.fields.keys())

        self._remove_fields()

        for base in reversed(self.cls.__mro__):
            self._set_polymorphic_base(base)

        for base in reversed(self.cls.__mro__):
            self._configure_polymorphism(base)

        add_class_to_registry(classname, self.cls)

    def _set_polymorphic_base(self, base):

        mapper_args = getattr(base, '__mapper_args__', {})
        is_polymorphic_base = 'polymorphic_on' in mapper_args

        if is_polymorphic_base:

            # User may set the polymorphic_on field using a field
            # instance or as a string.  As MapperConfig eventually removes
            # All the field attrs, we need to use the newly created cls.fields
            # dict otherwise _polymorphic_on will be unset when the fields
            # are removed due to garbage collection.
            field_or_field_name = mapper_args['polymorphic_on']
            if isinstance(field_or_field_name, Field):
                field_name = field_or_field_name.name
            else:
                field_name = field_or_field_name

            self.cls._polymorphic_base = True
            self.cls._polymorphic_opts = {
                'polymorphic_on': self.cls.fields.get(field_name),
                'allow_polymorphic_marshal':
                mapper_args.get('allow_polymorphic_marshal', False)
            }
            self.cls._polymorphic_identities = {}
        else:
            self.cls._polymorphic_base = False

    def _configure_polymorphism(self, base):

        mapper_args = getattr(base, '__mapper_args__', {})
        if 'polymorphic_name' in mapper_args:

            def _set_polymorphic_identity(parent, mapper):

                parent._polymorphic_identities.update({
                    mapper.__mapper_args__['polymorphic_name']:
                    mapper
                })

            # find the base polymorphic mapper
            for mapper in reversed(base.__mro__):
                if getattr(mapper, '_polymorphic_base', False):
                    _set_polymorphic_identity(mapper, base)
                    break

    def _remove_fields(self):
        """Cycle through the list of ``fields`` and remove those
        fields as attrs from the new cls being generated

        :returns: None
        """

        for name in self.cls.fields.keys():
            if getattr(self.cls, name, None):
                delattr(self.cls, name)

    def _extract_defined_pipes(self, base):
        """Extract, process and store pipes defined using the decorator syntax
        on this mapper

        """
        cls = self.cls
        _validators = {}
        _inputs = {}
        _processors = {}
        _outputs = {}

        for name, obj in vars(base).items():

            types = ['validation', 'process', 'output', 'input']
            hook_type = getattr(obj, '__mapper_field_hook', None)

            if not callable(obj) or hook_type not in types:
                continue
            elif hook_type == 'validation':
                for name in obj._field_names:
                    _validators.setdefault(name, [])
                    _validators[name].append(obj)
            elif hook_type == 'input':
                for name in obj._field_names:
                    _inputs.setdefault(name, [])
                    _inputs[name].append(obj)
            elif hook_type == 'process':
                for name in obj._field_names:
                    _processors.setdefault(name, [])
                    _processors[name].append(obj)
            elif hook_type == 'output':
                for name in obj._field_names:
                    _outputs.setdefault(name, [])
                    _outputs[name].append(obj)

        cls.defined_inputs = _inputs
        cls.defined_validators = _validators
        cls.defined_processors = _processors
        cls.defined_outputs = _outputs

    def _set_field_pipes(self, field, pipes, pipe_type):
        """Populate a :class:``Field`` compute chains with pipes
        extracted from mapper defenitions.

        """

        if field.name in pipes:

            for p in pipes[field.name]:
                opts = getattr(p, '__mapper_field_hook_opts', {})
                if opts['marshal']:
                    field.opts.extra_marshal_pipes[pipe_type] \
                        .append(pipe(p, **opts['pipe_opts']))
                if opts['serialize']:
                    field.opts.extra_serialize_pipes[pipe_type] \
                        .append(pipe(p, **opts['pipe_opts']))

            return field

    def _extract_fields(self, base):
        """Cycle over attrs declared on ``base`` searching for a types that
        inherit from :class:`kim.field.Field`.  If a field type is found, store
        it inside ``fields``.

        :param base: Current class from the MRO.
        :returns: None
        """

        cls = self.cls
        _fields = {}

        _fields.update(getattr(base, 'fields', {}))
        for name, obj in vars(base).items():

            # Add field to declared fields and remove cls.field
            if isinstance(obj, Field):
                obj.opts.set_name(attribute_name=name)

                _fields.update({name: obj})

                self._set_field_pipes(obj, cls.defined_inputs, 'input')
                self._set_field_pipes(obj, cls.defined_validators, 'validation')
                self._set_field_pipes(obj, cls.defined_processors, 'process')
                self._set_field_pipes(obj, cls.defined_outputs, 'output')

        cls.fields = OrderedDict(
            sorted(_fields.items(), key=lambda o: o[1]._creation_order))

    def _extract_roles(self, base):
        """update ``roles`` with any roles defined previously in
        the MRO and add any roles defined on the current
        ``base`` being iterated.

        Each base iterated in the MRO overwrites ``roles`` allowing
        users to inherit and override roles all the way up the inheritance
        chain.

        :param base: Current class from the MRO.
        :returns: None
        """

        cls = self.cls

        _roles = {}
        _roles.update(getattr(cls, 'roles', None) or {})
        _roles.update(getattr(base, '__roles__', None) or {})

        # Roles may be passed as list, convert to whitelist
        # objects in this case
        for name, role in six.iteritems(_roles):
            if isinstance(role, list):
                _roles[name] = whitelist(*role)
            elif not isinstance(role, Role):
                msg = "role %s on %s must be list or Role " \
                      "instance, got %s" % (name, self.__class__.__name__,
                                            type(role))
                raise MapperError(msg)

        cls.roles = _roles


# TODO(mike) __docs__
class MapperMeta(type):

    def __init__(cls, classname, bases, dict_):
        _MapperConfig.setup_mapping(cls, classname, dict_)
        type.__init__(cls, classname, bases, dict_)


class MapperSession(object):
    """Object that represents the state of a :class:`Mapper` during the execution of
    marshaling and serialization :class:`Pipeline`.
    """

    __slots__ = ('mapper', 'data', 'output', 'partial')

    def __init__(self, mapper, data, output, partial=None):
        """Instantiate a new instance of :class:`MapperSession`

        :param mapper: :class:`Mapper <Mapper>` instance.
        :param data: The data marshaled by the :class:`Mapper`
        :param output: The object the :class:`Mapper` is outputting  to.
        :return: None
        :rtype: None

        .. seealso::
            get_mapper_session method :func:`~Mapper.get_mapper_session`

        """

        self.mapper = mapper
        self.data = data
        self.output = output
        self.partial = partial


class Mapper(six.with_metaclass(MapperMeta, object)):
    """Mappers are the building blocks of Kim - they define how JSON output
    should look and how input JSON should be expected to look.

    Mappers consist of Fields. Fields define the shape and nature of the data
    both when being serialised(output) and marshaled(input).

    Mappers must define a ``__type__``. This is the type that will be
    instantiated if a new object is marshaled through the mapper. ``__type__``
    may be be any object that supports ``getattr`` and ``setattr``, or any dict
    like object.

    Usage::

        from kim import Mapper, field

        class UserMapper(Mapper):
            __type__ = User

            id = field.Integer(read_only=True)
            name = field.String(required=True)
            company = field.Nested('myapp.mappers.CompanyMapper')

    """

    #: The python type this Mapper will marshal to.
    __type__ = None
    "The python type this Mapper will marshal to."

    #: dictionary containing the role definitions for this mapper.
    __roles__ = {}

    @classmethod
    def many(cls, **mapper_params):
        """Provide access to a :class:`MapperIterator` to allow multiple
        items to be mapped by a mapper.

        :param mapper_params: dict of params passed to each new instance of the mapper.
        :return: :class:`MapperIterator <MapperIterator>` object
        :rtype: :class:`MapperIterator`

        Usage::

            >>> mapper = Mapper.many(data=data).marshal()
        """

        return MapperIterator(cls, **mapper_params)

    def __init__(self, obj=None, data=None, partial=False, raw=False,
                 parent=None):
        """Initialise a Mapper with the object and/or the data to be
        serialzed/marshaled. Mappers must be instantiated once per object/data.
        At least one of obj or data must be passed.

        :param obj: the object to be serialized, or updated by marshaling
        :param data: input data to be used for marshaling
        :param raw: the mapper will instruct fields to populate themselves
            using __dunder__ field names where required.
        :param partial: allow pipelines to pull data from an existing source
            or fall back to standard checks.
        :param parent: The parent of this Mapper.  Set internally when a Mapper
            is being used as a nested field.

        :raises: :class:`MapperError`
        :returns: None
        :rtype: None
        """

        if obj is None and data is None:
            raise MapperError(
                'At least one of obj or data must be passed to %s()'
                % self.__class__.__name__)

        self.obj = obj
        self.data = data
        self.errors = {}
        self.raw = raw
        self.partial = partial
        self.parent = parent

    @property
    def initial_errors(self):

        return getattr(self, '_initial_errors', None)

    def _get_mapper_type(self):
        """Return the specified type for this Mapper.  If no ``__type__`` is
        defined a :class:`MapperError` is raised

        :raises: :class:`MapperError`
        :returns: The specified ``__type__`` for the mapper.
        """

        if self.__type__ is None:
            raise MapperError(
                '%s must define a __type__' % self.__class__.__name__)

        return self.__type__

    def _get_obj(self):
        """Return ``self.obj`` or create a new instance of ``self.__type__``

        :returns: ``self.obj`` or new instance of ``self.__type__``
        """
        if self.obj is not None:
            return self.obj
        else:
            return self._get_mapper_type()()

    def _get_role(self, name_or_role, deferred_role=None):
        """Resolve a string to a role and check it exists, or check a
        directly passed role is a Role instance and return it.

        You may also affect the fields returned from a role at read time
        using ``deferred_role``.  deferred_role is used to provide the intersection
        between the role specified at ``name_or_role`` and the ``deferred_role``.

        Usage::

            class FooMapper(Mapper):
                __type__ = dict
                name = field.String()
                id = field.String()
                secret = field.String()

                __roles__ = {
                    'overview': whitelist('id', 'name'),
                }

            mapper._get_role('overview', deferred_role=whitelist('id'))

        Deferred roles can be used for things like allowing end users to provide a list
        of fields they want back from your API but only if they appear in a role you've
        specified.

        :param deferred_role: provide a role containing fields to dynamically change the
            permitted fields for the role specified in ``name_or_role``
        :param name_or_role: role name as a string or a Role instance
        :raises: :class:`MapperError`
        :returns: Role instance
        :rtype: :class:`Role <Role>`
        """
        if isinstance(name_or_role, six.string_types):
            try:
                role = self.roles[name_or_role]
            except KeyError:
                raise MapperError("Role '%s' not found on %s" % (
                                  name_or_role, self.__class__.__name__))
        elif isinstance(name_or_role, Role):
            role = name_or_role
        else:
            raise MapperError('role must be string or Role instance, got %s'
                              % type(name_or_role))

        # If deferred_role is not None, return the intersection of the
        # role and the deffered_role
        if deferred_role is not None:
            if not isinstance(deferred_role, Role):
                raise MapperError('deferred_role must be instance of Role')

            return role & deferred_role
        else:
            return role

    def _field_in_data(self, field):
        """Validate if a field.name appears in the provided data

        :param field: :class:`Field` instance
        :returns: Boolean
        :rtype: boolean
        """
        for key in self.data.keys():
            if key == field.name:
                return True
        return False

    def _get_fields(self, name_or_role, deferred_role=None, for_marshal=False):
        """Returns a list of :class:`Field` instances providing they are
        registered in the specified :class:`Role`.

        If the provided name_or_role is not found in the Mappers role list an
        error will be raised.

        :param deferred_role: an instance of role used to dynamically a new role.
        :param name_or_role: the name of a role as a string or a :class:`Role` instance.
        :param for_marshal: Indicate that the mapper is marshaling data.
        :raises: :class:`MapperError`
        :returns: list of :class:`Field <Field>` instances
        :rtype: list
        """

        role = self._get_role(name_or_role, deferred_role=deferred_role)

        fields = [f for name, f in six.iteritems(self.fields) if name in role]

        if self.partial and for_marshal:
            # If this is a partial update, rather than going through all fields
            # in the role, select those fields which are actually present in
            # the data - as long as they're also present in the role.
            return [f for f in fields if self._field_in_data(f)]
        else:
            return fields

    def _data_supports_transform(self, data):
        """return a boolean indicating if the given data object supports key
        based iteration

        :param data: the data object being used for iteration
        :returns: Boolean
        :rtype: boolean
        """

        return getattr(data, 'keys', False) is not False

    def _remove_none(self, output):
        """If all components of a dictionary are none, remove it and mark
        the entire data as None

        This method is used to evaluate data that has been transformed when
        the `raw=True` option is used.

        :param output: the data being evaulated
        :returns: If all values are none, None will be returned.  Otherwise ``output``
            will be returned with the ``None`` values removed.
        :rtype: mixed
        """
        all_none = True
        for k, v in six.iteritems(output):
            if type(v) == defaultdict:
                output[k] = self._remove_none(v)
            else:
                if v is not None:
                    all_none = False
        if all_none:
            return None
        else:
            return output

    def transform_data(self, data):
        """Transform a flat list of key names into a nested data structure by inflating
        dunder_score key name into objects.

        :param data: The object or data being transformed
        :returns: transformed data
        :rtype: dict

        Usage::

            >>> data = ['id', 'name', 'contact_details__phone',
                        'contact_details__address__postcode']
            >>> mapper.transform_data(data)
            {
                'id': x,
                'name': x,
                'contact_details': {
                    'phone': x,
                    'address': {
                        'postcode': x
                    }
                }
            }

        """
        if not self._data_supports_transform(data):
            raise MapperError('%s object does not appear to '
                              'support key based iteration '
                              'required when raw=True' % type(data))

        output = recursive_defaultdict()
        for key in data.keys():
            path = key.split('__')
            target = output
            # Walk along the path until we find the final dict to insert the
            # data into
            for component in path[:-1]:
                target = target[component]
            # And finally set the key on the final dict to the relevant data
            target[path[-1]] = getattr(data, key)

        return self._remove_none(output)

    def get_mapper_session(self, data, output):
        """Populate and return a new instance of :class:`MapperSession`

        :param data: data being Mapped
        :param output: obj mapper is mapping too
        :return: :class:`MapperSession <MapperSession>` object
        :rtype: :class:`MapperSession` object
        """

        return MapperSession(self, data, output, partial=self.partial)

    def serialize(self, role='__default__', raw=False, deferred_role=None):
        """Serialize ``self.obj`` into a dict according to the fields
        defined on this Mapper.

        :param role: specify the role to use when serializing this mapper
        :param raw: instruct the mapper to transform the data before serializing.
            This option overrides the Mapper.raw setting.
        :raises: :class:`FieldInvalid` :class:`MapperError`
        :returns: dict containing serialized object
        :rtype: mixed

        Usage::
            >>> mapper = UserMapper(obj=user)
            >>> mapper.serialize(role='public')

        .. seealso::
            :func:`~Mapper.transform_data`
        """

        output = {}  # Should this be user definable?

        if self.obj is None:
            raise MapperError(
                'Attmpted to serialize None, have you passed a valid obj param to %s()?'
                % self.__class__.__name__)

        data = self.obj
        transform_data = raw or self.raw
        if transform_data:
            data = self.transform_data(data)

        mapper_session = self.get_mapper_session(data, output)
        for field in self._get_fields(role, deferred_role=deferred_role):
            field.serialize(mapper_session)

        return output

    def marshal(self, role='__default__'):
        """Marshal ``self.data`` into ``self.obj`` according to the fields
        defined on this Mapper.

        :returns: Object of ``__type__`` populated with data
        """

        # Polymorphic mappers do some validation on incoming data.
        # if we have any initial_errors present, dont' bother continuing.
        if self.initial_errors is not None:
            raise MappingInvalid(self.initial_errors)

        output = self._get_obj()
        data = self.data

        fields = self._get_fields(role, for_marshal=True)

        for field in fields:
            try:
                field.marshal(self.get_mapper_session(data, output))
            except FieldInvalid as e:
                self.errors[field.name] = e.message
            except MappingInvalid as e:
                # handle errors from nested mappers.
                self.errors[field.name] = e.errors

        # Call top level mapper validator for validations involving more
        # than one field
        try:
            self.validate(output)
        except FieldInvalid as e:
            self.errors[e.field.name] = e.message
        except MappingInvalid as e:
            self.errors = e.errors

        if self.errors:
            raise MappingInvalid(self.errors)

        return output

    def validate(self, output):
        """Mappers may subclass this method to perform top-level validation
        on multiple related fields, raising `FieldInvalid` or `MappingInvalid`
        if any problems are found.

        :raises: FieldInvalid
        :raises: MappingInvalid
        """
        pass


class PolymorphicMapper(Mapper):
    """PolymorphicMappers build on the normal Mapper system to provide functionality for
    serializing and marshaling collections of different objects with different data
    structures.

    Usage::

        from kim import Mapper, field

        class ActivityMapper(PolymorphicMapper):

            __type__ = Activity

            id = field.String()
            name = field.String()
            object_type = field.String(choices=['event', 'task'])
            created_at = field.DateTime(read_only=True)

            __mapper_args__ = {
                'polymorphic_on': object_type,
            }


        class TaskMapper(ActivityMapper):

            __type__ = Task

            status = field.String(read_only=True)
            is_complete = field.Boolean()

            __mapper_args__ = {
                'polymorphic_name': 'task'
            }


        class EventMapper(ActivityMapper):

            __type__ = Event

            location = field.String(read_only=True)

            __mapper_args__ = {
                'polymorphic_name': 'event'
            }
    """

    @classmethod
    def is_polymorphic_base(cls):
        """Return a boolean indicating if this cls is the base type in the class hierarchy

        :returns: True if the class is the base type, otherwise False
        :rtype: boolean
        """

        return getattr(cls, '_polymorphic_base', False)

    @classmethod
    def _get_polymorphic_on(cls):
        """Return the :class:`kim.field.Field` defined in
        ``__mapper__args`` used for evaluating the type of the object.

        :returns: A field used to decide an object's type.
        :rtype: :class:`kim.field.Field``
        """

        return cls._polymorphic_opts['polymorphic_on']

    def __new__(cls, data=None, obj=None, *args, **kwargs):
        """Create an new instance of a Mapper using the polymorphic_on key defined in
        __mapper__opts__.

        :param data: datum being marshaled by the Mapper
        :param obj: obj being serialized by the Mapper
        :param args: Args passed to newly created Mapper
        :param kwargs: Kwargs passed to newly created Mapper
        :raises: :class:`kim.exception.FieldInvalid`
        :returns: :class:`kim.mapper.Mapper` indentified by polymorphic_indentity
        :rtype: :class:`kim.mapper.Mapper`
        """

        if (cls.is_polymorphic_base()
                and not kwargs.get('initial_errors', None)):

            try:
                key = cls.get_polymorphic_key(obj=obj, data=data)
                return cls.get_polymorphic_identity(key)(
                    data=data, obj=obj, *args, **kwargs)
            except FieldInvalid as e:
                initial_errors = {
                    cls._get_polymorphic_on().opts.source:
                    e.message
                }
                _obj = super(PolymorphicMapper, cls).__new__(cls)
                setattr(_obj, '_initial_errors', initial_errors)
                return _obj

        return super(PolymorphicMapper, cls).__new__(cls)

    @classmethod
    def get_polymorphic_key(cls, obj=None, data=None):
        """Return the value from obj when serializing or from data when marshaling
        for the polymorphic_on key.

        :param data: datum being marshaled by the Mapper
        :param obj: obj being serialized by the Mapper
        :returns: the polymorphic type name
        :rtype: str
        :raises: :class:`kim.exception.FieldInvalid` :class:`kim.exception.MappingInvalid`
        """

        field = cls._get_polymorphic_on()
        key_name = field.opts.source
        allow_create = cls._polymorphic_opts.get(
            'allow_polymorphic_marshal', False)

        key = attr_or_key(obj, key_name)
        if key is not None:
            return key

        key = attr_or_key(data, key_name)
        if key is not None and allow_create:
            return key
        elif key and not allow_create:
            raise MappingInvalid('PolymorphicMapper does not allow marshaling')
        else:
            raise field.invalid('required')

    @classmethod
    def get_polymorphic_identity(cls, key):
        """Return the polymorphic mapper stored at ``key``.

        :param key: The name of a polymoprhic indentity
        :raises: :class:`kim.exception.MapperError`
        :rtype: :class:`kim.mapper.Mapper`
        :returns: the Mapper stored against key
        """

        try:
            return cls._polymorphic_identities[key]
        except KeyError:
            raise MapperError('invalid polymorphic_identity %s'
                              ' is not a valid identity' % key)


class MapperIterator(object):
    """Provides a symmetric interface for Mapping many objects in one batch.

    A simple example would be seriaizing a list of User objects from a database
    query or other source.

    Usage::

        from kim import Mapper, field

        class UserMapper(Mapper):
            __type__ = User

            id = field.Integer(read_only=True)
            name = field.String(required=True)
            company = field.Nested('myapp.mappers.CompanyMapper')

        objs = User.query.all()
        results = UserMapper.many().serialize(objs)
    """

    def __init__(self, mapper, **mapper_params):
        """Constructs a new instance of a MapperIterator.

        :param mapper: a :class:`.Mapper` to map each item too.
        :param mapper_params: a dict of kwargs passed to each mapper
        """

        self.mapper = mapper
        self.mapper_params = mapper_params

    def get_mapper(self, data=None, obj=None):
        """Return a new instance of the provided mapper.

        :param data: provide the new mapper with data when marshaling
        :param obj: provide the new mapper with data when serializing

        :rtype: :class:`.Mapper`
        :returns: a new :class:`.Mapper`
        """

        self.mapper_params.update({
            'data': data,
            'obj': obj
        })
        return self.mapper(**self.mapper_params)

    def serialize(self, objs, role='__default__', deferred_role=None):
        """Serializes each item in ``objs`` creating a new mapper each time.

        :param objs: iterable of objects to serialize
        :param role: name of a role to use when serializing

        :returns: list of serialized objects
        """

        output = []  # TODO should this be user defined?
        for obj in objs:
            output.append(self.get_mapper(obj=obj).serialize(
                role=role,
                deferred_role=deferred_role))

        return output

    def marshal(self, data, role='__default__'):
        """Marshals each item in ``data`` creating a new mapper each time.

        :param objs: iterable of objects to marshal
        :param role: name of a role to use when marshaling

        :returns: list of marshaled objects
        """

        output = []  # TODO should this be user defined?
        for datum in data:
            output.append(self.get_mapper(data=datum).marshal(role=role))

        return output



from .base import (
    Pipeline, get_data_from_source, update_output_to_name, set_default)


class SerializePipeline(Pipeline):
    """SerializePipeline

    .. seealso::
        :func:`kim.pipelines.base.get_data_from_name`
        :func:`kim.pipelines.base.update_output_to_name`
    """

    input_pipes = [get_data_from_source]
    validation_pipes = []
    process_pipes = [set_default]
    output_pipes = [update_output_to_name]



# encoding: utf-8

import json
import pytest

from ..conftest import get_mapper_session

from kim.field import FieldInvalid, String
from kim.pipelines.base import Session
from kim.pipelines.string import is_valid_string


def test_is_valid_string_pipe():
    """test piping data through is_valid_string.
    """

    class InvalidString(object):

        def __str__(self):
            raise ValueError('invalid string')

    field = String(name='test')
    invalid_string = InvalidString()
    session = Session(field, invalid_string, {})

    with pytest.raises(FieldInvalid):
        is_valid_string(session)

    session.data = 'yes'
    assert is_valid_string(session) == 'yes'


def test_string_input():
    # TODO this requires fleshing out some more..

    field = String(name='name', required=True)

    output = {}
    mapper_session = get_mapper_session(
        data={'name': 'foo', 'email': 'mike@mike.com'}, output=output)

    field.marshal(mapper_session)
    assert output == {'name': 'foo'}


def test_string_output():
    # TODO this requires fleshing out some more..

    class Foo(object):
        name = 'value'

    field = String(name='name', required=True)

    output = {}
    mapper_session = get_mapper_session(obj=Foo(), output=output)
    field.serialize(mapper_session)
    assert output == {'name': 'value'}


def test_string_input_unicode():
    # TODO this requires fleshing out some more..

    field = String(name='name', required=True)

    output = {}
    mapper_session = get_mapper_session(data={'name': u'unicöde'}, output=output)
    field.marshal(mapper_session)
    assert output == {'name': u'unicöde'}


def test_marshal_read_only_string():

    field = String(name='name', read_only=True, required=True)

    output = {}
    mapper_session = get_mapper_session(
        data={'name': 'foo', 'email': 'mike@mike.com'}, output=output)
    field.marshal(mapper_session)
    assert output == {}


def test_is_valid_choice():

    field = String(name='type', choices=['one', 'two'])
    output = {}
    mapper_session = get_mapper_session(data={'type': 'three'}, output=output)
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    mapper_session = get_mapper_session(data={'type': 'one'}, output=output)
    field.marshal(mapper_session)
    assert output == {'type': 'one'}


def test_string_input_cast():

    field = String(name='name', required=True)

    output = {}
    mapper_session = get_mapper_session(data={'name': 123}, output=output)
    field.marshal(mapper_session)
    assert output == {'name': '123'}

def test_min_length_input():
    field = String(name='length', min=3)

    output = {}
    mapper_session = get_mapper_session(data={'length': 'o'}, output=output)
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    mapper_session = get_mapper_session(data={'length': 'four'}, output=output)
    field.marshal(mapper_session)
    assert output == {'length': 'four'}


def test_max_length_input():
    field = String(name='length', max=3)

    output = {}
    mapper_session = get_mapper_session(data={'length': 'seven'}, output=output)
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    mapper_session = get_mapper_session(data={'length': 'one'}, output=output)
    field.marshal(mapper_session)
    assert output == {'length': 'one'}


def test_min_max_length_input():
    field = String(name='length', min=3, max=5)

    output = {}
    mapper_session = get_mapper_session(data={'length': 'o'}, output=output)
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    mapper_session = get_mapper_session(data={'length': 'onemore'}, output=output)
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    mapper_session = get_mapper_session(data={'length': 'four'}, output=output)
    field.marshal(mapper_session)
    assert output == {'length': 'four'}


def test_blank_true():
    field = String(name='blank', blank=True)

    output = {}
    mapper_session = get_mapper_session(data={'blank': ''}, output=output)
    field.marshal(mapper_session)
    assert output == {'blank': ''}


def test_blank_false():
    field = String(name='blank', blank=False)

    output = {}
    mapper_session = get_mapper_session(data={'blank': ''}, output=output)
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    mapper_session = get_mapper_session(data={'blank': 'bla'}, output=output)
    field.marshal(mapper_session)
    assert output == {'blank': 'bla'}


def test_string_input_cast_object():
    """If I pass a nested object instead of a string, it should be cast to a
    string"""

    field = String(name='name', required=True)

    output = {}
    mapper_session = get_mapper_session(
        data={'name': {'nested': 'stuff'}}, output=output)

    field.marshal(mapper_session)
    assert output == {'name': "{'nested': 'stuff'}"}


def test_supports_unicode():

    field = String(name='unicode')
    output = {}
    data = {'unicode': u"foo \u2192"}
    mapper_session = get_mapper_session(data=data, output=output)
    field.marshal(mapper_session)
    assert output == {'unicode': u'foo →'}


def test_supports_unicode_from_json():

    field = String(name='unicode')
    output = {}
    data = '{"unicode": "foo \u2192"}'
    mapper_session = get_mapper_session(data=json.loads(data), output=output)
    field.marshal(mapper_session)
    assert output == {'unicode': u'foo →'}



import pytest
import decimal

from kim.field import FieldInvalid, Integer, Decimal, Float
from kim.pipelines.base import Session
from kim.pipelines.numeric import is_valid_integer, is_valid_decimal

from ..conftest import get_mapper_session


def test_is_valid_integer_pipe():
    """test piping data through is_valid_integer.
    """

    field = Integer(name='test')
    session = Session(field, 'test', {})

    with pytest.raises(FieldInvalid):
        is_valid_integer(session)

    session.data = '2'
    assert is_valid_integer(session) == 2
    session.data = 2
    assert is_valid_integer(session) == 2
    session.data = 2.3
    assert is_valid_integer(session) == 2


def test_integer_input():

    field = Integer(name='name', required=True)

    mapper_session = get_mapper_session(
        data={'email': 'mike@mike.com'}, output={})
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    mapper_session = get_mapper_session(
        data={'name': 'foo', 'email': 'mike@mike.com'}, output={})
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    output = {}
    mapper_session = get_mapper_session(
        data={'name': 2, 'email': 'mike@mike.com'}, output=output)
    field.marshal(mapper_session)
    assert output == {'name': 2}

    output = {}
    mapper_session = get_mapper_session(
        data={'name': '2', 'email': 'mike@mike.com'}, output=output)
    field.marshal(mapper_session)
    assert output == {'name': 2}


def test_integer_field_invalid_type():

    field = Integer(name='name')
    mapper_session = get_mapper_session(
        data={'name': None, 'email': 'mike@mike.com'}, output={})
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)


def test_integer_output():

    class Foo(object):
        name = 2

    field = Integer(name='name', required=True)

    output = {}
    mapper_session = get_mapper_session(obj=Foo(), output=output)
    field.serialize(mapper_session)
    assert output == {'name': 2}


def test_marshal_read_only_integer():

    field = Integer(name='name', read_only=True, required=True)

    output = {}
    mapper_session = get_mapper_session(
        data={'id': 2, 'email': 'mike@mike.com'}, output=output)

    field.marshal(mapper_session)
    assert output == {}


def test_marshal_default():

    field = Integer(name='name', default=5)

    output = {}
    mapper_session = get_mapper_session(data={}, output=output)

    field.marshal(mapper_session)
    assert output == {'name': 5}


def test_is_valid_choice():

    field = Integer(name='type', choices=[1, 2])
    output = {}
    mapper_session = get_mapper_session(data={'type': 3}, output=output)
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    mapper_session = get_mapper_session(data={'type': 1}, output=output)
    field.marshal(mapper_session)
    assert output == {'type': 1}


def test_min_max_integer():

    field = Integer(name='age', min=20, max=35)
    output = {}

    mapper_session = get_mapper_session(data={'age': 15}, output=output)
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    mapper_session = get_mapper_session(data={'age': 40}, output=output)
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    mapper_session = get_mapper_session(data={'age': 25}, output=output)
    field.marshal(mapper_session)
    assert output == {'age': 25}

    mapper_session = get_mapper_session(data={'age': '25'}, output=output)
    field.marshal(mapper_session)
    assert output == {'age': 25}

    mapper_session = get_mapper_session(data={'age': 35}, output=output)
    field.marshal(mapper_session)
    assert output == {'age': 35}


def test_min_max_decimal():
    field = Decimal(name='age', min=0, max=1.5)
    output = {}

    mapper_session = get_mapper_session(data={'age': -1}, output=output)
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    mapper_session = get_mapper_session(data={'age': 4}, output=output)
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    mapper_session = get_mapper_session(data={'age': 0.7}, output=output)
    field.marshal(mapper_session)
    assert output == {'age': decimal.Decimal('0.7')}

    mapper_session = get_mapper_session(data={'age': 1.5}, output=output)
    field.marshal(mapper_session)
    assert output == {'age': decimal.Decimal('1.5')}


def test_is_valid_decimal_pipe():
    """test piping data through is_valid_decimal.
    """

    field = Decimal(name='test')
    session = Session(field, 'test', {})

    with pytest.raises(FieldInvalid):
        is_valid_decimal(session)

    session.data = '2.5'
    assert is_valid_decimal(session) == decimal.Decimal('2.5')
    session.data = 2
    assert is_valid_decimal(session) == decimal.Decimal(2)
    session.data = 2.3
    assert is_valid_decimal(session) == decimal.Decimal(2.3)


def test_decimal_input():

    field = Decimal(name='name', required=True)

    mapper_session = get_mapper_session(
        data={'email': 'mike@mike.com'}, output={})
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    mapper_session = get_mapper_session(
        data={'name': 'foo', 'email': 'mike@mike.com'}, output={})
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    output = {}
    mapper_session = get_mapper_session(
        data={'name': 2, 'email': 'mike@mike.com'}, output=output)
    field.marshal(mapper_session)
    assert output == {'name': decimal.Decimal(2)}


def test_decimal_input_precision():

    field = Decimal(name='name', required=True, precision=4)

    output = {}
    mapper_session = get_mapper_session(
        data={'name': '3.147261', 'email': 'mike@mike.com'}, output=output)
    field.marshal(mapper_session)
    assert output == {'name': decimal.Decimal('3.1473')}


def test_decimal_field_invalid_type():

    field = Decimal(name='name')
    mapper_session = get_mapper_session(
        data={'name': None, 'email': 'mike@mike.com'}, output={})
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)


def test_decimal_output():

    class Foo(object):
        name = decimal.Decimal('2.52')

    field = Decimal(name='name', required=True)

    output = {}
    mapper_session = get_mapper_session(obj=Foo(), output=output)
    field.serialize(mapper_session)
    assert output == {'name': '2.52000'}


def test_float_input():

    field = Float(name='name', required=True)

    mapper_session = get_mapper_session(
        data={'email': 'mike@mike.com'}, output={})
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    mapper_session = get_mapper_session(
        data={'name': 'foo', 'email': 'mike@mike.com'}, output={})
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    output = {}
    mapper_session = get_mapper_session(
        data={'name': 2, 'email': 'mike@mike.com'}, output=output)
    field.marshal(mapper_session)
    assert output == {'name': 2.0}


def test_min_max_float():

    field = Float(name='age', min=0, max=1.5)
    output = {}

    mapper_session = get_mapper_session(data={'age': -1}, output=output)
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    mapper_session = get_mapper_session(data={'age': 4}, output=output)
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)

    mapper_session = get_mapper_session(data={'age': 0.7}, output=output)
    field.marshal(mapper_session)
    assert output == {'age': 0.7}

    mapper_session = get_mapper_session(data={'age': 1.5}, output=output)
    field.marshal(mapper_session)
    assert output == {'age': 1.5}


def test_float_input_precision():

    field = Float(name='name', required=True, precision=4)

    output = {}
    mapper_session = get_mapper_session(
        data={'name': '3.147261', 'email': 'mike@mike.com'}, output=output)
    field.marshal(mapper_session)
    assert output == {'name': 3.1473}


def test_float_field_invalid_type():

    field = Decimal(name='name')

    mapper_session = get_mapper_session(
        data={'name': None, 'email': 'mike@mike.com'}, output={})
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)


def test_float_output():

    class Foo(object):
        name = 2.52056

    field = Float(name='name', required=True, precision=2)

    output = {}
    mapper_session = get_mapper_session(obj=Foo(), output=output)
    field.serialize(mapper_session)
    assert output == {'name': '2.52'}




from datetime import datetime, date
from iso8601.iso8601 import Utc

import pytest

from kim.field import FieldInvalid, DateTime, Date
from kim.pipelines.base import Session
from kim.pipelines.datetime import is_valid_datetime

from ..conftest import get_mapper_session


def test_is_valid_datetime_pipe():
    """test piping data through is_valid_datetime.
    """

    field = DateTime(name='test')
    session = Session(field, 'test', {})

    with pytest.raises(FieldInvalid):
        is_valid_datetime(session)

    session.data = '2016-02-29T12:00:12Z'
    assert is_valid_datetime(session) == \
        datetime(2016, 2, 29, 12, 0, 12, tzinfo=Utc())
    session.data = '2015-06-29T08:00:12Z'
    assert is_valid_datetime(session) == \
        datetime(2015, 6, 29, 8, 0, 12, tzinfo=Utc())


def test_datetime_input_default_format():

    field = DateTime(name='datetime', required=True)

    output = {}
    mapper_session = get_mapper_session(
        data={'datetime': '2015-06-29T08:00:12Z', 'email': 'mike@mike.com'},
        output=output)
    field.marshal(mapper_session)
    assert output == {'datetime': datetime(2015, 6, 29, 8, 0, 12, tzinfo=Utc())}


def test_datetome_input_custom_format_invalid_value():

    field = DateTime(name='datetime', format_str='%Y-%m-%d %H:%M:%S', required=True)

    mapper_session = get_mapper_session(
        data={'datetime': 'bla', 'email': 'mike@mike.com'},
        output={})
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)


def test_datetome_input_default_format_invalid_value():

    field = DateTime(name='datetime', required=True)

    mapper_session = get_mapper_session(
        data={'datetime': 'bla', 'email': 'mike@mike.com'},
        output={})
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)



def test_datetime_field_invalid_type():

    field = DateTime(name='date')
    mapper_session = get_mapper_session(
        data={'date': None, 'email': 'mike@mike.com'}, output={})
    with pytest.raises(FieldInvalid):
        field.marshal(mapper_session)


def test_datetime_output():

    class Foo(object):
        date = datetime(2015, 6, 29, 8, 0, 12, tzinfo=Utc())

    field = DateTime(name='date', required=True)

    output = {}
    mapper_session = get_mapper_session(obj=Foo(), output=output)
    field.serialize(mapper_session)
    assert output == {'date': '2015-06-29T08:00:12+00:00'}


def test_marshal_read_only_datetime():

    field = DateTime(name='date', read_only=True, required=True)

    output = {}
    mapper_session = get_mapper_session(
        data={'date': '2015-06-29T08:00:12Z', 'email': 'mike@mike.com'},
        output=output)

    field.marshal(mapper_session)
    assert output == {}


def test_date_input_default_format():

    field = Date(name='date', required=True)

    output = {}
    mapper_session = get_mapper_session(
        data={'date': '2015-06-29', 'email': 'mike@mike.com'},
        output=output)
    field.marshal(mapper_session)
    assert output == {'date': date(2015, 6, 29)}


def test_date_input_custom_format():

    field = Date(name='date', format_str='%Y', required=True)

    output = {}
    mapper_session = get_mapper_session(
        data={'date': '2015', 'email': 'mike@mike.com'},
        output=output)
    field.marshal(mapper_session)
    assert output == {'date': date(2015, 1, 1)}


def test_date_input_custom_format_invalid_format():

    field = Date(name='date', format_str='%Y', required=True)

    output = {}
    with pytest.raises(FieldInvalid):
        mapper_session = get_mapper_session(
            data={'date': 'bla', 'email': 'mike@mike.com'},
            output=output)
        field.marshal(mapper_session)


def test_date_output_default_format():

    class Foo(object):
        date = date(2015, 6, 29)

    field = Date(name='date', required=True)

    output = {}
    mapper_session = get_mapper_session(obj=Foo(), output=output)
    field.serialize(mapper_session)
    assert output == {'date': '2015-06-29'}


def test_date_output_custom_format():

    class Foo(object):
        date = date(2015, 6, 29)

    field = Date(name='date', format_str='%Y', required=True)

    output = {}
    mapper_session = get_mapper_session(obj=Foo(), output=output)
    field.serialize(mapper_session)
    assert output == {'date': '2015'}



from .base import Pipeline, read_only, get_data_from_name, update_output_to_source


class MarshalPipeline(Pipeline):
    """MarshalPipeline

    .. seealso::
        :func:`kim.pipelines.base.read_only`
        :func:`kim.pipelines.base.get_data_from_name`
        :func:`kim.pipelines.base.update_output_to_source`
    """

    input_pipes = [read_only, get_data_from_name]
    validation_pipes = []
    process_pipes = []
    output_pipes = [update_output_to_source]



# kim
# Copyright (C) 2014-2015 the Kim authors and contributors
# <see AUTHORS file>
#
# This module is part of Kim and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php


__version__ = '1.0.0'


from .mapper import Mapper, PolymorphicMapper
from .exception import (
    MapperError, MappingInvalid, RoleError, FieldOptsError, FieldError,
    FieldInvalid, StopPipelineExecution)
from .role import blacklist, whitelist
from .pipelines import pipe
from .field import (
    Field, String, Integer, Decimal, Boolean, Nested, Collection, Static,
    DateTime, Date)


__all__ = [
    Mapper, PolymorphicMapper, MapperError, MappingInvalid, RoleError,
    FieldOptsError, FieldError, FieldInvalid, StopPipelineExecution, blacklist,
    whitelist, pipe, Field, String, Integer, Decimal, Boolean, Nested,
    Collection, Static, DateTime, Date]
